#include <gtk/gtk.h>
#include <xmms/plugin.h>

static EffectPlugin pitch_ep;
static gint speed = 100;

static GtkWidget *window, *scrollbar;
static GtkObject *adjustment;



#define DO_PITCH(type, length, newlength, buffer, newbuffer) \
{ \
	gint delta = (length << 16) / newlength, i, off; \
	type *ptr = buffer, *newptr = newbuffer; \
	for(i = 0, off = 0; i < newlength; i++, off += delta) \
		*newptr++ = ptr[off >> 16]; \
}

static void adjustment_cb(GtkObject *obj, gpointer data)
{
	speed = 200 - (gint)GTK_ADJUSTMENT(adjustment)->value;
}

static void pitch_configure(void)
{
	if(window)
		return;
	window = gtk_window_new(GTK_WINDOW_DIALOG);
	gtk_signal_connect(GTK_OBJECT(window), "destroy",
			   GTK_SIGNAL_FUNC(gtk_widget_destroyed),
			   &window);
	gtk_container_set_border_width(GTK_CONTAINER(window), 5);
	adjustment = gtk_adjustment_new(100.0, 50, 150.0, 10.0, 10.0, 10.0);
	scrollbar = gtk_vscrollbar_new(GTK_ADJUSTMENT(adjustment));
	gtk_widget_set_usize(scrollbar, -1, 100);
	gtk_signal_connect(GTK_OBJECT(adjustment), "value_changed",
			   GTK_SIGNAL_FUNC(adjustment_cb), NULL);
	gtk_container_add(GTK_CONTAINER(window), scrollbar);
	gtk_widget_show(scrollbar);
	gtk_widget_show(window);
}
	       
static int pitch_mod_samples(gpointer *data, gint length, AFormat fmt,
			     gint srate, gint nch)
{
	gint bps = 1, newlength;
	static gpointer new_buffer = NULL;
	static gint new_buffer_length = 0;
	
	
	if(fmt == FMT_S16_LE || fmt == FMT_S16_BE || fmt == FMT_S16_NE ||
	   fmt == FMT_U16_LE || fmt == FMT_U16_BE || fmt == FMT_U16_NE)
		bps *= 2;
	bps *= nch;
	length /= bps;
	newlength = (length * 100) / speed;
	if((newlength * bps) > new_buffer_length)
	{
		new_buffer_length = newlength * bps;
		new_buffer = g_realloc(new_buffer, new_buffer_length);
	}
	switch(bps)
	{
	case 1:
		DO_PITCH(gint8, length, newlength, *data, new_buffer);
		break;
	case 2:
		DO_PITCH(gint16, length, newlength, *data, new_buffer);
		break;
	case 4:
		DO_PITCH(gint32, length, newlength, *data, new_buffer);
		break;
	}		
	*data = new_buffer;
	return newlength * bps;
}

EffectPlugin *get_eplugin_info(void)
{
	pitch_ep.description = "Pitch Control";
	pitch_ep.configure = pitch_configure;
	pitch_ep.mod_samples = pitch_mod_samples;

	return &pitch_ep;
}

